﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Net;
using System.Net.NetworkInformation;
using System.Net.Sockets;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Monnit;
using Monnit.Mine;


namespace SampleMineApplication
{
    public partial class MainForm : Form
    {
        private MineServer _Server;

        public MainForm()
        {
            InitializeComponent();
        }

        private void OnLoad(object sender, EventArgs e)
        {
            LoadIPAddresses();
            ddlProtocol.DataSource = Enum.GetValues(typeof(BaseApplication.eMineListenerProtocol));
            ddlGatewayType.DataSource = Enum.GetValues(typeof(BaseApplication.eGatewayType));
            ddlApplication.DataSource = Enum.GetValues(typeof(BaseApplication.eSensorApplication));
			ddlSensorGen.DataSource = Enum.GetValues(typeof(BaseApplication.eFirmwareGeneration));
        }

        private void btnStart_Click(object sender, EventArgs e)
        {
            int port;
            if (!int.TryParse(txtPort.Text, out port))
            {
                MessageBox.Show("Invalid Port");
            }
            else
            {
                BaseApplication.eMineListenerProtocol Protocol;
                Enum.TryParse<BaseApplication.eMineListenerProtocol>(ddlProtocol.SelectedValue.ToString(), out Protocol);

                btnStart.Enabled = false;
                ddlIpSelector.Enabled = false;
                ddlProtocol.Enabled = false;
                txtPort.Enabled = false;
				_Server = new MineServer(Protocol, (IPAddress)ddlIpSelector.SelectedItem, port);

				/* Use this MineServer constructor to use your own implementation of ExternalDataAccess.
				 */
				//iDataAccess dataAccess = new ExternalDataAccess();
				//_Server = new MineServer(Protocol, (IPAddress)ddlIpSelector.SelectedItem, port, dataAccess);

                _Server.StartServer();
                PrintOutput("Server has been started");

                _Server.LogException += Handle_LogException;
                _Server.PersistGateway += Handle_PersistGateway;
                _Server.PersistSensor += Handle_PersistSensor;
                _Server.SensorMessage += Handle_SensorMessage;
                _Server.GatewayMessage += Handle_GatewayMessage;
                _Server.UnknownGateway += Handle_UnknownGateway;
            }
        }

        private void btnRegisterGateway_Click(object sender, EventArgs e)
        {
            UInt32 GatewayID = ValidateGateway();
            if (GatewayID > 0)
            {
                try
                {
                    BaseApplication.eGatewayType GatewayType;
                    Enum.TryParse<BaseApplication.eGatewayType>(ddlGatewayType.SelectedValue.ToString(), out GatewayType);

					// The values are just example values, be sure to change them according to what you need
					Monnit.Mine.Gateway MineGateway = new Monnit.Mine.Gateway(GatewayID, GatewayType, "3.2.1.0", "2.3.0.0", "127.0.0.1", Convert.ToInt32(txtPort.Text));
                    
                    _Server.RegisterGateway(MineGateway);
					//MineGateway.ForceConfiguration();
                    PrintOutput(string.Format("GatewayID {0} has been registered", GatewayID));
                }
                catch(Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }

        private void btnFindGateway_Click(object sender, EventArgs e)
        {
            UInt32 GatewayID = ValidateGateway();
            if (GatewayID > 0)
            {
                Monnit.Mine.Gateway GatewayObj = _Server.FindGateway(GatewayID);
                if (GatewayObj == null)
                {
                    lbOutput.Items.Insert(0, string.Format("GatewayID {0} has not been registered", GatewayID));
                }
                else
                {
                    MineGateway gatewayForm = new MineGateway(GatewayObj);
                    gatewayForm.Show();
                }
            }
        }

        private void btnRemoveGateway_Click(object sender, EventArgs e)
        {
            UInt32 GatewayID = ValidateGateway();
            if (GatewayID > 0)
            {
                if (_Server.FindGateway(GatewayID) == null)
                {
                    MessageBox.Show(string.Format("Gateway {0} has not been registered", GatewayID));
                }
                else
                {
                    _Server.RemoveGateway(GatewayID);
                    PrintOutput(string.Format("GatewayID {0} has been removed", GatewayID));
                }
            }
        }

        private void btnRegisterSensor_Click(object sender, EventArgs e)
        {
            UInt32 SensorID = ValidateSensor();
            UInt32 GatewayID = ValidateGateway();
            if (SensorID > 0)
            {
                try
                {
					string generation = ddlSensorGen.SelectedValue.ToString();

					switch (generation)
					{
						case "Alta":
							generation = "GEN2";
							break;
						case "Wifi":
							generation = "WIFI";
							break;
						case "Commercial":
						default:
							generation = "GEN1";
							break;
					}

                    BaseApplication.eSensorApplication SensorApplication;
                    Enum.TryParse<BaseApplication.eSensorApplication>(ddlApplication.SelectedValue.ToString(), out SensorApplication);

                    Monnit.Mine.Sensor MineSensor = new Monnit.Mine.Sensor(SensorID, SensorApplication, "2.3.0.0", generation);
                    _Server.RegisterSensor(GatewayID, MineSensor);

                    PrintOutput(string.Format("SensorID {0} has been registered", SensorID));
                }
                catch(Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }

        private void btnFindSensor_Click(object sender, EventArgs e)
        {
            UInt32 SensorID = ValidateSensor();
            if (SensorID > 0)
            {
                Monnit.Mine.Sensor SensorObj = _Server.FindSensor(SensorID);
                if (SensorObj == null)
                {
                    PrintOutput(string.Format("SensorID {0} has not been registered", SensorID));
                }
                else
                {
                    MineSensor sensorForm = new MineSensor(SensorObj);
                    sensorForm.Show();
                }
            }
        }

        private void btnRemoveSensor_Click(object sender, EventArgs e)
        {
            UInt32 SensorID = ValidateSensor();
            if (SensorID > 0)
            {
                if (_Server.FindSensor(SensorID) == null)
                {
                    MessageBox.Show(string.Format("SensorID {0} has not been registered", SensorID));
                }
                else
                {
                    _Server.RemoveSensor(SensorID);
                    PrintOutput(string.Format("SensorID {0} has been removed", SensorID));
                }
            }
        }

        private void btnClear_Click(object sender, EventArgs e)
        {
            lbOutput.Items.Clear();
        }

		private void reformBtn_Click(object sender, EventArgs e)
		{
            UInt32 GatewayID = ValidateGateway();
            if (GatewayID > 0)
			{
				Gateway GatewayObj = _Server.FindGateway(GatewayID);
				if (GatewayObj != null)
				{
					GatewayObj.ReformNetwork();
					PrintOutput("Reform pending.");
				}
				else
				{
					PrintOutput("Gateway does not exist.");
				}
			}
		}

		private void pointBtn_Click(object sender, EventArgs e)
		{
            UInt32 GatewayID = ValidateGateway();
            if (GatewayID > 0)
			{
				Gateway GatewayObj = _Server.FindGateway(GatewayID);
				if (GatewayObj != null)
				{
					GatewayObj.ServerHostAddress = "u1.sensorsgateway.com";
					GatewayObj.Port = 3000;
					GatewayObj.IsDirty = true;

					PrintOutput("update pending.");
				}
				else
				{
					PrintOutput("Gateway does not exist.");
				}
			}
		}

        private void LoadIPAddresses()
        {
            ddlIpSelector.Items.Clear();
            ddlIpSelector.Items.Add(IPAddress.Any);
            ddlIpSelector.SelectedIndex = 0;

            NetworkInterface[] interfaces = NetworkInterface.GetAllNetworkInterfaces();
            IPHostEntry host;
            host = Dns.GetHostEntry(Dns.GetHostName());

            foreach (NetworkInterface adapter in interfaces)
            {
                if (adapter.OperationalStatus == OperationalStatus.Up
                    && (adapter.NetworkInterfaceType == NetworkInterfaceType.Ethernet
                        || adapter.NetworkInterfaceType == NetworkInterfaceType.Wireless80211
                        || adapter.NetworkInterfaceType == NetworkInterfaceType.GigabitEthernet))
                {
                    var ipProps = adapter.GetIPProperties();

                    foreach (var ip in ipProps.UnicastAddresses)
                    {
                        if (ip.Address.AddressFamily == AddressFamily.InterNetwork)
                        {
                            //adapter.Description.ToString() + " - " + ip.Address.ToString();
                            ddlIpSelector.Items.Add(ip.Address);
                        }
                    }
                }
            }
        }

        private UInt32 ValidateGateway()
        {
            if (_Server == null)
            {
                MessageBox.Show("Server not started");
                return 0;
            }

            UInt32 GatewayID;
            if (!UInt32.TryParse(txtGateway.Text, out GatewayID))
            {
                MessageBox.Show("Invalid GatewayID");
                return 0;
            }

            return GatewayID;
        }

        private UInt32 ValidateSensor()
        {
            if (_Server == null)
            {
                MessageBox.Show("Server not started");
                return 0;
            }

            UInt32 SensorID;
            if (!UInt32.TryParse(txtSensor.Text, out SensorID))
            {
                MessageBox.Show("Invalid SensorID");
                return 0;
            }

            return SensorID;
        }

        private void PrintOutput(string[] output)
        {
            this.BeginInvoke(new Action(() =>
            {
                for (int i = output.Length - 1; i >= 0; i--)
                {
                    lbOutput.Items.Insert(0, output[i]);
                }
                while (lbOutput.Items.Count > 500)
                    lbOutput.Items.RemoveAt(500);
            }));

        }
    
        private void PrintOutput(string output)
        {
            this.BeginInvoke(new Action(() =>
            {
                lbOutput.Items.Insert(0, output);

                while (lbOutput.Items.Count > 500)
                    lbOutput.Items.RemoveAt(500);
            }));
                
        }

        void Handle_UnknownGateway(object sender, UnknownGatewayEventArgs e)
        { 
             PrintOutput(string.Format("Unknown  GatewayID: {0}, GatewayType: {1}",e.GatewayID,e.GatewayType.ToString()));
        }

        void Handle_GatewayMessage(object sender, GatewayMessageEventArgs e)
        {
			PrintOutput(string.Format("{0} GatewayID: {1} Type:{2} MessageCount: {3}", e.GatewayMessage.ReceivedDate, e.GatewayMessage.GatewayID, (BaseApplication.eGatewayMessageType)e.GatewayMessage.MessageType, e.GatewayMessage.MessageCount));
        }

        void Handle_SensorMessage(object sender, SensorMessagesEventArgs e)
        {
			List<string> OutputList = new List<string>();
            foreach (SensorMessage message in e.SensorMessages)
            {
                OutputList.Add(string.Format("{1} SensorID: {0} State: {4} Signal: {3}% Battery: {2}v", message.SensorID, message.MessageDate, message.Voltage, message.SignalStrengthPercent(), message.State));
                foreach (Datum datum in message.Data)
                    OutputList.Add(string.Format("     {0}: {1}", datum.Description, datum.Data));
            }
            PrintOutput(OutputList.ToArray());
        }

        void Handle_PersistSensor(object sender, HandlePersistSensorEventArgs e)
        {
            PrintOutput(string.Format("SensorID {0} was updated in memory.", e.Sensor.SensorID));
        }

        void Handle_PersistGateway(object sender, HandlePersistGatewayEventArgs e)
        {
            PrintOutput(string.Format("GatewayID {0} was updated in memory.", e.Gateway.GatewayID));
        }

        void Handle_LogException(object sender, HandleLogExceptionEventArgs e)
        {
            PrintOutput(string.Format("Exception - {0}", e.Exception.Message));
        }

        private void updatebtn_Click(object sender, EventArgs e)
        {
            string sensorid = string.IsNullOrWhiteSpace(txtSensor.Text) ? "0" : txtSensor.Text;
            int id = Convert.ToInt32(sensorid);
            if (_Server != null)
            {
                Monnit.Mine.Sensor sensor = _Server.FindSensor(Convert.ToUInt32(id));
                if (sensor != null)
                {
					string generation = (ddlSensorGen.SelectedValue.ToString());

					switch (generation)
					{
						case "Alta":
							generation = "GEN2";
							break;
						case "Wifi":
							generation = "WIFI";
							break;
						case "Commercial":
						default:
							generation = "GEN1";
							break;
					}
					sensor.GenerationType = generation;
                    UpdateTemperatureSensor f = new UpdateTemperatureSensor(sensor);
                    f.Show();
                }
                else
                {
                    MessageBox.Show("Could not find sensor using sensorID. Make sure Temperature Sensor is registered");
                }
            }
            else
            {
                MessageBox.Show("please create your server and register sensors before updating.");
            }
        }
    }
}