﻿using Monnit;
using Monnit.Mine;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.IO;

namespace SampleMineApplication
{
    public class ExternalDataAccess : IDataAccess
    {
        public Gateway FindGateway(uint gatewayID)
		{
            Gateway gateway = null;

            string jsonFilePath = @"" + ConfigurationManager.AppSettings["JsonFilePath"];
            string json = File.ReadAllText(jsonFilePath);

            try
            {
                JObject jObject = JObject.Parse(json);

                if (jObject != null)
                {
                    JArray gatewayArray = (JArray)jObject["Gateways"];

                    if (gatewayArray != null)
                    {
                        foreach (var item in gatewayArray)
                        {
                            uint fileGatewayID = Convert.ToUInt32(item["ID"].ToString());
                            if(gatewayID == fileGatewayID)
                            {
                                BaseApplication.eGatewayType gatewayType = (BaseApplication.eGatewayType)Enum.Parse(typeof(BaseApplication.eGatewayType), item["Type"].ToString());
                                string firmwareVersion = item["FirmwareVersion"].ToString();
                                string radioFirmwareVersion = item["RadioFirmwareVersion"].ToString();
                                string host = item["Host"].ToString();
                                int port = Convert.ToInt32(item["Port"].ToString());
                                double reportInterval = Convert.ToDouble(item["ReportInterval"].ToString());
                                //byte[] encryptionKey = JsonConvert.DeserializeObject<byte[]>(item["EncryptionKey"].ToString());
                                //byte[] encryptionIV = JsonConvert.DeserializeObject<byte[]>(item["EncryptionIV"].ToString());
                                byte[] encryptionKey = ExtentionMethods.StringToByteArray(item["EncryptionKey"].ToString());
                                byte[] encryptionIV = ExtentionMethods.StringToByteArray(item["EncryptionIV"].ToString());

                                if (encryptionKey != null && encryptionIV != null)
                                {
                                    gateway = new Gateway(gatewayID, gatewayType, firmwareVersion, radioFirmwareVersion, host, port, encryptionKey, encryptionIV);
                                }
                                else
                                {
                                    gateway = new Gateway(gatewayID, gatewayType, firmwareVersion, radioFirmwareVersion, host, port);
                                }
                                gateway.ReportInterval = reportInterval;
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("----------------FindGateway-------------------");
                Console.WriteLine("--Message--");
                Console.WriteLine(ex.Message);
                Console.WriteLine("--Stacktrace--");
                Console.WriteLine(ex.StackTrace);
                Console.WriteLine("----------------------------------------------");
            }

            return gateway;
		}

		public Sensor FindSensor(uint sensorID)
		{
            Sensor sensor = null;

            string jsonFilePath = @"" + ConfigurationManager.AppSettings["JsonFilePath"];
            string json = File.ReadAllText(jsonFilePath);

            try
            {
                JObject jObject = JObject.Parse(json);

                if (jObject != null)
                {
                    JObject sensorJObject = (JObject)jObject["Sensor_" + sensorID];
                    BaseApplication.eSensorApplication sensorType = (BaseApplication.eSensorApplication)Enum.Parse(typeof(BaseApplication.eSensorApplication), sensorJObject["Type"].ToString());
                    string sensorFirmwareVersion = sensorJObject["FirmwareVersion"].ToString();
                    string sensorPlatform = sensorJObject["Platform"].ToString();
                    double sensorHeartbeat = Convert.ToDouble(sensorJObject["ReportInterval"].ToString());

                    sensor = new Sensor(sensorID, sensorType, sensorFirmwareVersion, sensorPlatform, sensorHeartbeat);
                    //_Sensors[sensorID] = sensor;
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("----------------FindSensor-------------------");
                Console.WriteLine("--Message--");
                Console.WriteLine(ex.Message);
                Console.WriteLine("--Stacktrace--");
                Console.WriteLine(ex.StackTrace);
                Console.WriteLine("----------------------------------------------");
            }

            return sensor;
        }

        public List<Sensor> FindSensorsByGateway(uint gatewayID)
        {
            List<Sensor> gatewaySensors = new List<Sensor>();

            string jsonFilePath = @"" + ConfigurationManager.AppSettings["JsonFilePath"];
            string json = File.ReadAllText(jsonFilePath);

            try
            {
                JObject jObject = JObject.Parse(json);

                if (jObject != null)
                {
                    JArray gatewayArray = (JArray)jObject["Gateways"];

                    if (gatewayArray != null)
                    {
                        foreach (var item in gatewayArray)
                        {
                            uint fileGatewayID = Convert.ToUInt32(item["ID"].ToString());
                            if (gatewayID == fileGatewayID)
                            {
                                JArray sensorArray = (JArray)item["SensorIDList"];
                                //_AssignedSensors[gateway.GatewayID] = new List<uint>();

                                if (sensorArray != null)
                                {
                                    foreach (var sensorItem in sensorArray)
                                    {
                                        uint sensorID = Convert.ToUInt32(sensorItem["ID"].ToString());
                                        JObject sensorJObject = (JObject)jObject["Sensor_" + sensorID];
                                        BaseApplication.eSensorApplication sensorType = (BaseApplication.eSensorApplication)Enum.Parse(typeof(BaseApplication.eSensorApplication), sensorJObject["Type"].ToString());
                                        string sensorFirmwareVersion = sensorJObject["FirmwareVersion"].ToString();
                                        string sensorPlatform = sensorJObject["Platform"].ToString();
                                        double sensorHeartbeat = Convert.ToDouble(sensorJObject["ReportInterval"].ToString());

                                        Sensor sensor = new Sensor(sensorID, sensorType, sensorFirmwareVersion, sensorPlatform, sensorHeartbeat);
                                        //_Sensors[sensorID] = sensor;
                                        //_AssignedSensors[gateway.GatewayID].Add(sensorID);
                                        gatewaySensors.Add(sensor);
                                    }
                                }
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("----------------FindSensorsByGateway-------------------");
                Console.WriteLine("--Message--");
                Console.WriteLine(ex.Message);
                Console.WriteLine("--Stacktrace--");
                Console.WriteLine(ex.StackTrace);
                Console.WriteLine("----------------------------------------------");
            }

            return gatewaySensors;
        }

        public void LogException(Exception ex, string location)
		{
            Console.WriteLine("----------------- Exception -----------------");
            Console.WriteLine(location + ": " + ex.Message);
            Console.WriteLine(location + ": " + ex.StackTrace);
            Console.WriteLine("---------------------------------------------");
        }

        public void UpdateGatewayEncryption(Gateway gateway)
        {
            //Gateway externalGateway = FindGateway(gateway.GatewayID);
            if(gateway != null)
            {
                //externalGateway.EncryptionKey = gateway.EncryptionKey;
                //externalGateway.EncryptionIV = gateway.EncryptionIV;
                uint gatewayID = gateway.GatewayID;
                string jsonFilePath = @"" + ConfigurationManager.AppSettings["JsonFilePath"];
                string json = File.ReadAllText(jsonFilePath);

                try
                {
                    JObject jObject = JObject.Parse(json);

                    if (jObject != null)
                    {
                        JArray gatewayArray = (JArray)jObject["Gateways"];

                        if (gatewayArray != null)
                        {
                            foreach (var item in gatewayArray)
                            {
                                uint fileGatewayID = Convert.ToUInt32(item["ID"].ToString());
                                if (gatewayID == fileGatewayID)
                                {
                                    //item["EncryptionKey"] = JsonConvert.SerializeObject(gateway.EncryptionKey);
                                    //item["EncryptionIV"] = JsonConvert.SerializeObject(gateway.EncryptionIV);
                                    item["EncryptionKey"] = ExtentionMethods.ByteArrayToString(gateway.EncryptionKey);
                                    item["EncryptionIV"] = ExtentionMethods.ByteArrayToString(gateway.EncryptionIV);
                                }
                            }

                            jObject["Gateways"] = gatewayArray;
                            string output = JsonConvert.SerializeObject(jObject, Formatting.Indented);
                            File.WriteAllText(jsonFilePath, output);
                        }
                    }
                }
                catch (Exception ex)
                {
                    Console.WriteLine("----------------FindSensorsByGateway-------------------");
                    Console.WriteLine("--Message--");
                    Console.WriteLine(ex.Message);
                    Console.WriteLine("--Stacktrace--");
                    Console.WriteLine(ex.StackTrace);
                    Console.WriteLine("----------------------------------------------");
                }

                //_GatewayKeys[gateway.GatewayID] = gateway.EncryptionKey;
                //_GatewayIVs[gateway.GatewayID] = gateway.EncryptionIV;
            }
        }
	}
}